---
title: "Time-to-Event Analyses"
output:
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Time-to-Event Analyses}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This article is web-only - please access it through the User Guides section of
the [admiral website](https://pharmaverse.github.io/admiral/cran-release/).
